---
title: Best Practices
summary: Rules which enforce generally accepted best practices.
permalink: pmd_rules_plsql_bestpractices.html
folder: pmd/rules/plsql
sidebaractiveurl: /pmd_rules_plsql.html
editmepath: ../pmd-plsql/src/main/resources/category/plsql/bestpractices.xml
keywords: Best Practices, TomKytesDespair
language: PLSQL
---
<!-- DO NOT EDIT THIS FILE. This file is generated from file ../pmd-plsql/src/main/resources/category/plsql/bestpractices.xml. -->
## TomKytesDespair

**Since:** PMD 5.1

**Priority:** Medium (3)

&quot;WHEN OTHERS THEN NULL&quot; hides all errors - (Re)RAISE an exception or call RAISE_APPLICATION_ERROR

**This rule is defined by the following XPath expression:**
``` xpath
//ExceptionHandler[QualifiedName/@Image='OTHERS' and upper-case(Statement/UnlabelledStatement/Expression/@Image)='NULL']
```

**Example(s):**

``` sql
{%raw%}CREATE OR REPLACE PACKAGE BODY update_planned_hrs
IS

PROCEDURE set_new_planned (p_emp_id IN NUMBER, p_project_id IN NUMBER, p_hours IN NUMBER)
IS
BEGIN
   UPDATE employee_on_activity ea
   SET ea.ea_planned_hours = p_hours
   WHERE
            ea.ea_emp_id = p_emp_id
            AND ea.ea_proj_id = p_project_id;

EXCEPTION
          WHEN NO_DATA_FOUND THEN
          RAISE_APPLICATION_ERROR (-20100, 'No such employee or project');

END set_new_planned;

FUNCTION existing_planned (p_emp_id IN NUMBER, p_project_id IN NUMBER) RETURN NUMBER

IS

existing_hours NUMBER(4);

BEGIN
   SELECT ea.ea_planned_hours INTO existing_hours
   FROM employee_on_activity ea
   WHERE
            ea.ea_emp_id = p_emp_id
            AND ea.ea_proj_id = p_project_id;

   RETURN (existing_hours);

   EXCEPTION
          WHEN OTHERS THEN NULL;

   END existing_planned;

END update_planned_hrs;
/{%endraw%}
```

**Use this rule by referencing it:**
``` xml
<rule ref="category/plsql/bestpractices.xml/TomKytesDespair" />
```

